@@ -1,5 +1,8 @@
 use crate::*;
 
+#[cfg(feature = "serde")]
+use serde_test::{assert_de_tokens, assert_ser_tokens, Token};
+
 #[test]
 fn parser() {
     // ensure that constant init works
@@ -349,16 +352,80 @@ fn parser() {
         )
     );
 
-    parse(include_bytes!("testdata/line-daemon-end.txt"), false).unwrap();
-    parse(include_bytes!("testdata/line-netfilter.txt"), false).unwrap();
-    parse(include_bytes!("testdata/line-anom-abend.txt"), false).unwrap();
-    parse(include_bytes!("testdata/line-anom-abend-2.txt"), false).unwrap();
-    parse(include_bytes!("testdata/line-user-auth.txt"), false).unwrap();
-    parse(include_bytes!("testdata/line-sockaddr-unix.txt"), false).unwrap();
-    parse(include_bytes!("testdata/line-sockaddr-unix-2.txt"), false).unwrap();
-    parse(include_bytes!("testdata/line-user-auth-2.txt"), false).unwrap();
-    parse(include_bytes!("testdata/line-mac-policy-load.txt"), false).unwrap();
-    parse(include_bytes!("testdata/line-tty.txt"), false).unwrap();
+    for enriched in [true, false] {
+        for split_msg in [true, false] {
+            for (n, line) in [
+                &include_bytes!("testdata/line-acct-lock.txt")[..],
+                &include_bytes!("testdata/line-add-group.txt")[..],
+                &include_bytes!("testdata/line-add-user.txt")[..],
+                &include_bytes!("testdata/line-anom-abend-2.txt")[..],
+                &include_bytes!("testdata/line-anom-abend.txt")[..],
+                &include_bytes!("testdata/line-avc-denied.txt")[..],
+                &include_bytes!("testdata/line-avc-granted.txt")[..],
+                &include_bytes!("testdata/line-bpf.txt")[..],
+                &include_bytes!("testdata/line-broken-avc-info.txt")[..],
+                &include_bytes!("testdata/line-broken-subj1.txt")[..],
+                &include_bytes!("testdata/line-broken-subj2.txt")[..],
+                &include_bytes!("testdata/line-chgrp-id.txt")[..],
+                &include_bytes!("testdata/line-cred-acq.txt")[..],
+                &include_bytes!("testdata/line-cred-disp.txt")[..],
+                &include_bytes!("testdata/line-cred-refr.txt")[..],
+                &include_bytes!("testdata/line-crypto-key-user.txt")[..],
+                &include_bytes!("testdata/line-crypto-param-change-user.txt")[..],
+                &include_bytes!("testdata/line-crypto-session.txt")[..],
+                &include_bytes!("testdata/line-daemon-end-2.txt")[..],
+                &include_bytes!("testdata/line-daemon-end.txt")[..],
+                &include_bytes!("testdata/line-daemon-start.txt")[..],
+                &include_bytes!("testdata/line-del-group.txt")[..],
+                &include_bytes!("testdata/line-del-user.txt")[..],
+                &include_bytes!("testdata/line-eoe.txt")[..],
+                &include_bytes!("testdata/line-execve.txt")[..],
+                &include_bytes!("testdata/line-grp-mgmt.txt")[..],
+                &include_bytes!("testdata/line-mac-policy-load.txt")[..],
+                &include_bytes!("testdata/line-netfilter.txt")[..],
+                &include_bytes!("testdata/line-netlabel.txt")[..],
+                &include_bytes!("testdata/line-path-enriched.txt")[..],
+                &include_bytes!("testdata/line-path.txt")[..],
+                &include_bytes!("testdata/line-sockaddr-unix-2.txt")[..],
+                &include_bytes!("testdata/line-sockaddr-unix.txt")[..],
+                &include_bytes!("testdata/line-sockaddr-unknown-1.txt")[..],
+                &include_bytes!("testdata/line-sockaddr-unknown-2.txt")[..],
+                &include_bytes!("testdata/line-sockaddr-unknown-3.txt")[..],
+                &include_bytes!("testdata/line-software-update.txt")[..],
+                &include_bytes!("testdata/line-syscall.txt")[..],
+                &include_bytes!("testdata/line-tty.txt")[..],
+                &include_bytes!("testdata/line-unknown.txt")[..],
+                &include_bytes!("testdata/line-uringop.txt")[..],
+                &include_bytes!("testdata/line-user-acct.txt")[..],
+                &include_bytes!("testdata/line-user-auth-2.txt")[..],
+                &include_bytes!("testdata/line-user-auth.txt")[..],
+                &include_bytes!("testdata/line-user-avc-1.txt")[..],
+                &include_bytes!("testdata/line-user-avc-2.txt")[..],
+                &include_bytes!("testdata/line-user-chauthtok.txt")[..],
+                &include_bytes!("testdata/line-user-end.txt")[..],
+                &include_bytes!("testdata/line-user-err.txt")[..],
+                &include_bytes!("testdata/line-user-login.txt")[..],
+                &include_bytes!("testdata/line-user-logout.txt")[..],
+                &include_bytes!("testdata/line-user-mgmt.txt")[..],
+                &include_bytes!("testdata/line-user-role-change.txt")[..],
+                &include_bytes!("testdata/line-user-selinux-err.txt")[..],
+                &include_bytes!("testdata/line-user-start.txt")[..],
+                &include_bytes!("testdata/line-usys-config.txt")[..],
+            ]
+            .iter()
+            .enumerate()
+            {
+                Parser {
+                    enriched,
+                    split_msg,
+                }
+                .parse(line)
+                .unwrap_or_else(|_| {
+                    panic!("failed to parse {n} (enriched={enriched}, split_msg={split_msg}")
+                });
+            }
+        }
+    }
 }
 
 #[test]
@@ -367,7 +434,7 @@ fn test_msg_kv() {
         split_msg: true,
         ..Parser::default()
     };
-    for line in [
+    for (n, line) in [
         &include_bytes!("testdata/line-acct-lock.txt")[..],
         &include_bytes!("testdata/line-add-group.txt")[..],
         &include_bytes!("testdata/line-add-user.txt")[..],
@@ -398,17 +465,24 @@ fn test_msg_kv() {
         &include_bytes!("testdata/line-user-avc-1.txt")[..],
         &include_bytes!("testdata/line-user-avc-2.txt")[..],
         &include_bytes!("testdata/line-user-selinux-err.txt")[..],
-    ] {
-        let m = p.parse(line).unwrap();
-        println!("{:?}", &m);
-        let msg = m
-            .body
+    ]
+    .iter()
+    .enumerate()
+    {
+        let Message {
+            node: _,
+            ty: _,
+            id,
+            body,
+        } = p.parse(line).unwrap();
+        println!("test {n}: {id}: {body:?}");
+        let msg = body
             .get("msg")
-            .unwrap_or_else(|| panic!("{}: Field msg not found", m.id));
+            .unwrap_or_else(|| panic!("test {n}: {id}: Field msg not found"));
         match msg {
             Value::Map(_) => {}
-            Value::Str(_, _) => panic!("{}: Field msg was parsed as string", m.id),
-            _ => panic!("{}: Field msg was parsed as something else", m.id),
+            Value::Str(_, _) => panic!("test {n}: {id}: Field msg was parsed as string"),
+            _ => panic!("test {n}: {id}: Field msg was parsed as something else"),
         }
     }
 }
@@ -431,3 +505,188 @@ fn breakage_sockaddr_unknown() {
     )
     .expect("can't parse line-sockaddr-unknown-3.txt");
 }
+
+#[test]
+#[cfg(feature = "serde")]
+fn serde_messagetype() {
+    assert_ser_tokens(&MessageType::SYSCALL, &[Token::String("SYSCALL")]);
+    assert_ser_tokens(&MessageType(20000), &[Token::String("UNKNOWN[20000]")]);
+}
+
+#[test]
+#[cfg(feature = "serde")]
+fn serde_key() {
+    for (obj, tok) in &[
+        (&Key::Name(b"foo"[..].into()), &[Token::String("foo")]),
+        (&Key::NameUID(b"euid"[..].into()), &[Token::String("euid")]),
+        (&Key::NameGID(b"egid"[..].into()), &[Token::String("egid")]),
+        (&Key::Common(Common::Arch), &[Token::String("arch")]),
+        (&Key::Arg(1, None), &[Token::String("a1")]),
+        (&Key::Arg(2, Some(3)), &[Token::String("a2[3]")]),
+        (&Key::ArgLen(2), &[Token::String("a2_len")]),
+    ] {
+        assert_ser_tokens(obj, *tok);
+        assert_de_tokens(*obj, *tok);
+    }
+
+    for (obj, tok) in &[
+        (
+            &Key::NameTranslated(b"foo"[..].into()),
+            &[Token::String("FOO")],
+        ),
+        (&Key::Literal("foo"), &[Token::String("foo")]),
+    ] {
+        assert_ser_tokens(obj, *tok);
+    }
+}
+
+#[test]
+#[cfg(feature = "serde")]
+fn serde_value() {
+    for q in &[Quote::None, Quote::Single, Quote::Double] {
+        assert_ser_tokens(&Value::Str(&b"foo"[..], *q), &[Token::Bytes(b"foo")]);
+    }
+    assert_ser_tokens(
+        &Value::Str(&b"foo"[..], Quote::Braces),
+        &[Token::Bytes(b"{foo}")],
+    );
+
+    for (obj, tok) in &[
+        (Value::Empty, &[Token::Unit][..]),
+        (Value::Owned(b"foo".to_vec()), &[Token::Bytes(b"foo")]),
+        (Value::Number(Number::Hex(16)), &[Token::String("0x10")]),
+        (Value::Number(Number::Oct(16)), &[Token::String("0o20")]),
+        (Value::Number(Number::Dec(16)), &[Token::I64(16)]),
+        (
+            Value::List(vec![]),
+            &[Token::Seq { len: Some(0) }, Token::SeqEnd][..],
+        ),
+        (
+            Value::List(vec![
+                Value::Owned(b"foo".to_vec()),
+                Value::Owned(b"bar".to_vec()),
+                Value::Owned(b"baz".to_vec()),
+                Value::from(42),
+            ]),
+            &[
+                Token::Seq { len: Some(4) },
+                Token::Bytes(b"foo"),
+                Token::Bytes(b"bar"),
+                Token::Bytes(b"baz"),
+                Token::I64(42),
+                Token::SeqEnd,
+            ][..],
+        ),
+        (
+            Value::Map(vec![]),
+            &[Token::Map { len: Some(0) }, Token::MapEnd][..],
+        ),
+        (
+            Value::Map(vec![(
+                Key::Name(b"foo"[..].into()),
+                Value::Owned(b"bar".to_vec()),
+            )]),
+            &[
+                Token::Map { len: Some(1) },
+                Token::String("foo"),
+                Token::Bytes(b"bar"),
+                Token::MapEnd,
+            ][..],
+        ),
+    ] {
+        assert_ser_tokens(obj, &tok[..]);
+        assert_de_tokens(obj, &tok[..]);
+    }
+}
+
+#[test]
+#[cfg(feature = "serde")]
+fn serde_number() {
+    for (n, t) in &[
+        (Number::Dec(10), Token::I64(10)),
+        (Number::Hex(0x10), Token::String("0x10")),
+        (Number::Oct(0o10), Token::String("0o10")),
+    ] {
+        assert_ser_tokens(n, &[*t]);
+        assert_de_tokens(n, &[*t]);
+    }
+    assert_de_tokens(&Number::Dec(10), &[Token::U64(10)]);
+}
+
+#[test]
+#[cfg(feature = "serde")]
+fn serde_event_id() {
+    let obj = EventID {
+        timestamp: 1615225617302,
+        sequence: 25836,
+    };
+    let tok = Token::String("1615225617.302:25836");
+
+    assert_ser_tokens(&obj, &[tok]);
+    assert_de_tokens(&obj, &[tok]);
+}
+
+#[test]
+#[cfg(feature = "serde")]
+fn serde_message_type() {
+    for (m, t) in &[
+        (MessageType::SYSCALL, Token::String("SYSCALL")),
+        (MessageType(9999), Token::String("UNKNOWN[9999]")),
+    ] {
+        assert_ser_tokens(m, &[*t]);
+        assert_de_tokens(m, &[*t]);
+    }
+}
+
+#[test]
+#[cfg(feature = "serde")]
+fn serde_message() {
+    let msg = parse(include_bytes!("testdata/line-eoe.txt"), false).unwrap();
+    assert_ser_tokens(&msg.body, &[Token::Map { len: Some(0) }, Token::MapEnd]);
+
+    let msg = parse(include_bytes!("testdata/line-execve.txt"), false).unwrap();
+    assert_ser_tokens(
+        &msg.body,
+        &[
+            Token::Map { len: Some(2) },
+            Token::String("argc"),
+            Token::I64(0),
+            Token::String("a0"),
+            Token::Bytes(b"whoami"),
+            Token::MapEnd,
+        ],
+    );
+}
+
+#[test]
+fn parse_uringop() {
+    let msg = parse(include_bytes!("testdata/line-uringop.txt"), false).unwrap();
+    println!("{msg:?}");
+    let v = msg
+        .body
+        .get("uring_op")
+        .unwrap_or_else(|| panic!("{}: uring_op not found", msg.id));
+    assert_eq!(*v, Value::Number(Number::Dec(18)));
+}
+
+#[test]
+fn parse_bpf() {
+    let msg = parse(include_bytes!("testdata/line-bpf.txt"), false).unwrap();
+    println!("{msg:?}");
+    let v = msg
+        .body
+        .get("prog-id")
+        .unwrap_or_else(|| panic!("{}: prog-id not found", msg.id));
+    assert_eq!(*v, Value::Number(Number::Dec(75)));
+}
+
+#[test]
+
+fn special() {
+    Parser {
+        enriched: false,
+        split_msg: false,
+    }
+    .parse(&include_bytes!("testdata/line-daemon-start.txt")[..])
+    .unwrap_or_else(|e| panic!("{e}"));
+}
